;; DS1620a.asm - DS1620 High Reolution Temperature Measurement Sensor

include "ds1620a.inc"

;
ThermPort                  :equ 00h     ; SysPort0
ThermMaskBits              :equ 07h     ;
ThermData                  :equ 01h     ;
ThermClock                 :equ 02h     ;
ThermReset                 :equ 04h     ;

gbThermProtocol            :equ 30h     ;
gbThermPortValue           :equ 31h     ;
gbThermPortMirror          :equ 32h     ;
gbThermTempRead            :equ 33h     ;
gbThermTempRead2           :equ 34h     ;
gbThermTempLast            :equ 78h     ;USBEndP1FIFO
gbThermTempLast2           :equ 79h     ;USBEndP1FIFO +1

;//$PAGE
;*******************************************************************************
; ThermInitialize()
; @func Initialize the thermometer to continuous mode.
;*******************************************************************************
ThermInitialize:

    ; Standalone mode
    mov  a,ThermConfigRead
    call ThermPortRead

    ; Check mode
    mov  a,[gbThermPortValue]
    and  a,ThermControlOneShot | ThermControlCPUUse
    cmp  a,ThermControlCPUUse
    jz   ThermInitDone

    ; Set mode
    mov  a,[gbThermPortValue]
    and  a,~(ThermControlOneShot | ThermControlCPUUse)
    or   a,ThermControlCPUUse | 08h ; set reserved bit
    mov  [gbThermPortValue],a

    ; Write it out
    mov  a,ThermConfigWrite
    call ThermPortWrite

    ; Wait 10 milliseconds
    mov  a,10
    call SysDelayMS

  ThermInitDone:
    ; Start conversion
    mov  a,ThermConvertStart
    mov  [gbThermProtocol],a
    call ThermPortResetHigh
    call ThermPortProtocolWrite
    call ThermPortResetLow

    ret

;//$PAGE
;*******************************************************************************
; ThermReadTemperature()
; @func Read the current temperature.
;*******************************************************************************
ThermReadTemperature:

    ; Initialize results
    mov  a,0
    mov  [gbThermTempRead],a
    mov  [gbThermTempRead2],a

    ; Read Temperature request
    mov  a,ThermTempRead
    mov  [gbThermProtocol],a

    ; Get temperature
    call ThermPortResetHigh
    call ThermPortProtocolWrite
    call ThermPortReadTemperature
    call ThermPortResetLow

    ; Save results
    mov  a,[gbThermTempRead]
    mov  [gbThermTempLast],a
    mov  a,[gbThermTempRead2]
    mov  [gbThermTempLast2],a

    ret

;//$PAGE
;*******************************************************************************
; ThermPortResetHigh()
; @func .
;*******************************************************************************
ThermPortResetHigh:

    ; Initialize mirror
    iord [ThermPort]
    mov  [gbThermPortMirror],a

    ; ThermReset =1;
    or   a,ThermReset
    mov  [gbThermPortMirror],a
    iowr ThermPort
    ret

;*******************************************************************************
; ThermPortResetLow()
; @func .
;*******************************************************************************
ThermPortResetLow:
    ; ThermReset =0;
    mov  a,[gbThermPortMirror]
    and  a,~ThermReset
    iowr ThermPort
    ret

;*******************************************************************************
; ThermWaitForDone()
; @func Wait for indication of temperature conversion complete.
;*******************************************************************************
ThermWaitForDone:

    ; Read config
    mov  a,ThermConfigRead
    call ThermPortRead

    ; Test flag
    mov  a,[gbThermPortValue]
    and  a,ThermControlDone
    jz   ThermWaitForDone
    ret

;*******************************************************************************
; ThermPortWrite()
; @func Write protocol byte and value byte.
; @parm byte | A | Protocol to send.
;*******************************************************************************
ThermPortWrite:

    ; Save input
    mov  [gbThermProtocol],a

    call ThermPortResetHigh
    call ThermPortProtocolWrite
    call ThermPortWrite8Bits
    call ThermPortResetLow
    ret

;*******************************************************************************
; ThermPortRead()
; @func Write protocol byte and read value byte into gbThermPortValue.
; @parm byte | A | Protocol to send.
;*******************************************************************************
ThermPortRead:

    ; Save input
    mov  [gbThermProtocol],a

    ; Initialize results
    mov  a,0
    mov  [gbThermPortValue],a

    call ThermPortResetHigh
    call ThermPortProtocolWrite
    call ThermPortRead8Bits
    call ThermPortResetLow
    ret

;*******************************************************************************
; ThermPortProtocolWrite()
; @func .
; @parm BYTE | gbThermProtocol | Protocol value.
;*******************************************************************************
ThermPortProtocolWrite:

    mov  a,[gbThermPortValue]
    push a
    mov  a,[gbThermProtocol]
    mov  [gbThermPortValue],a
    call ThermPortWrite8Bits
    pop  a
    mov  [gbThermPortValue],a

    ret

;//$PAGE
;*******************************************************************************
; ThermPortReadTemperature()
; @func .
; @parm BYTE | gbThermTempRead | Returned read temperature (low bits).
; @parm BYTE | gbThermTempRead2 | Returned read temperature (high bit).
;*******************************************************************************
ThermPortReadTemperature:

    ; Setup bitmask
    mov  a,1
    push a
    pop  x

    mov  a,8
  ThermPortTempReadLoop:
    push a
    ;*********************************************    

    ; Tri-state data pin for input
    mov  a,[gbThermPortMirror]
    or   a,ThermData
    iowr ThermPort

    ; ThermClock =0;
    mov  a,[gbThermPortMirror]
    and  a,~ThermClock
    mov  [gbThermPortMirror],a
    iowr ThermPort

    ; Read in data pin and check for 0 or 1
    iord ThermPort
    and  a,ThermData
    jnz  ThermPortTempReadValue1

  ThermPortTempReadValue0:
    jmp  ThermPortTempReadClock
  ThermPortTempReadValue1:
    ; Use bitmask
    push x
    pop  a
    or   [gbThermTempRead],a
   ;jmp  ThermPortTempReadClock ; redundant, but good practice

  ThermPortTempReadClock:
    ; ThermClock =1;
    mov  a,[gbThermPortMirror]
    or   a,ThermClock
    mov  [gbThermPortMirror],a
    iowr ThermPort

    ; Next bit in mask
    push x
    pop  a
    asl  a
    push a
    pop  x

    ;*********************************************    
    ; Finished?
    pop  a
    dec  a
    jnz  ThermPortTempReadLoop

    ;*********************************************    
    ; Last bit
    ;*********************************************    
    ; Tri-state data pin for input
    mov  a,[gbThermPortMirror]
    or   a,ThermData
    iowr ThermPort

    ; ThermClock =0;
    mov  a,[gbThermPortMirror]
    and  a,~ThermClock
    mov  [gbThermPortMirror],a
    iowr ThermPort

    ; Read in data pin and check for 0 or 1
    iord ThermPort
    and  a,ThermData
    jnz  ThermPortTempReadLastValue1

  ThermPortTempReadLastValue0:
    jmp  ThermPortTempReadLastClock
  ThermPortTempReadLastValue1:
    ; Use bitmask
    mov  a,1
    mov  [gbThermTempRead2],a
   ;jmp  ThermPortTempReadLastClock ; redundant, but good practice

  ThermPortTempReadLastClock:
    ; ThermClock =1;
    mov  a,[gbThermPortMirror]
    or   a,ThermClock
    mov  [gbThermPortMirror],a
    iowr ThermPort

    ret

;//$PAGE
;*******************************************************************************
; ThermPortRead8Bits()
; @func .
; @parm BYTE | gbThermPortValue | Return read byte.
;*******************************************************************************
ThermPortRead8Bits:

    ; Setup bitmask
    mov  a,1
    push a
    pop  x

    mov  a,8
 ThermPortValueReadLoop:
    push a
    ;*********************************************    

    ; Tri-state data pin for input
    mov  a,[gbThermPortMirror]
    or   a,ThermData
    iowr ThermPort

    ; ThermClock =0;
    mov  a,[gbThermPortMirror]
    and  a,~ThermClock
    mov  [gbThermPortMirror],a
    iowr ThermPort

    ; Read in data pin and check for 0 or 1
    iord ThermPort
    and  a,ThermData
    jnz  ThermPortReadValue1

  ThermPortReadValue0:
    jmp  ThermPortReadClock
  ThermPortReadValue1:
    ; Use bitmask
    push x
    pop  a
    or   [gbThermPortValue],a
   ;jmp  ThermPortReadClock ; redundant, but good practice

  ThermPortReadClock:
    ; ThermClock =1;
    mov  a,[gbThermPortMirror]
    or   a,ThermClock
    mov  [gbThermPortMirror],a
    iowr ThermPort

    ; Next bit in mask
    push x
    pop  a
    asl  a
    push a
    pop  x

    ;*********************************************    
    ; Finished?
    pop  a
    dec  a
    jnz  ThermPortValueReadLoop

    ret

;//$PAGE
;*******************************************************************************
; ThermPortWrite8Bits()
; @func .
; @parm BYTE | gbThermPortValue | Value to write
;*******************************************************************************
ThermPortWrite8Bits:

    ; Setup bitmask
    mov  a,1
    push a
    pop  x

    mov  a,8
  ThermPortWriteLoop:
    push a
    ;*********************************************    

    ; Get bitmask
    push x
    pop  a

    and  a,[gbThermPortValue]
    jnz  ThermPortWriteValue1

  ThermPortWriteValue0:
    ; ThermData =0;
    mov  a,[gbThermPortMirror]
    and  a,~ThermData
    mov  [gbThermPortMirror],a
    iowr ThermPort
    jmp  ThermPortWriteValueClock
  ThermPortWriteValue1:
    ; ThermData =1;
    mov  a,[gbThermPortMirror]
    or   a,ThermData
    mov  [gbThermPortMirror],a
    iowr ThermPort
   ;jmp  ThermPortWriteValueClock ; redundant, but good practice

  ThermPortWriteValueClock:
    ; ThermClock =0;
    mov  a,[gbThermPortMirror]
    and  a,~ThermClock
    mov  [gbThermPortMirror],a
    iowr ThermPort

    ; Next bit in mask
    push x
    pop  a
    asl  a
    push a
    pop  x

    ; ThermClock =1;
    mov  a,[gbThermPortMirror]
    or   a,ThermClock
    iowr ThermPort

    ;*********************************************    
    ; Finished?
    pop  a
    dec  a
    jnz  ThermPortWriteLoop

    ret
