unit EZUSB2;

interface
uses Windows;

type _ANCHOR_DOWNLOAD_CONTROL = record
   Offset: Word;
end;

type _GET_STRING_DESCRIPTOR_IN = record
 Index: Byte;
 LanguageId: Word;
end;

type _SET_INTERFACE_IN = record
 interfaceNum: Byte;
 alternateSetting: Byte;
end;

type _BULK_TRANSFER_CONTROL = record
 pipeNum: Longword;
end;

type _ISO_TRANSFER_CONTROL = record
 PipeNum: Longword;
 PacketSize: Longword;
 PacketCount: Longword;
 FramesPerBuffer: Longword;
 BufferCount: Longword;
end;

type _USBD_ISO_PACKET_DESCRIPTOR = record
 Offset: Longword;
 Length: Longword;
 Status: Byte ;    {....USBD_STATUS   ??}
end;

type _VENDOR_OR_CLASS_REQUEST_CONTROL = record
 direction: Byte;
 requestType: Byte;
 recepient: Byte;
 requestTypeReservedBits: Byte;
 request: Byte;
 value: Word;
 index: Word;
end;



type _USB_DEVICE_DESCRIPTOR = record
    bLength: Byte;
    bDescriptorType: Byte;
    bcdUSB: Word;
    bDeviceClass: Byte;
    bDeviceSubClass: Byte;
    bDeviceProtocol: Byte;
    bMaxPacketSize0: Byte;
    idVendor: Word;
    idProduct: Word;
    bcdDevice: Word;
    iManufacturer: Byte;
    iProduct: Byte;
    iSerialNumber: Byte;
    bNumConfigurations: Byte;
end;

Type _USB_ENDPOINT_DESCRIPTOR = record
    bLength: Byte;
    bDescriptorType: Byte;
    bEndpointAddress: Byte;
    bmAttributes: Byte;
    wMaxPacketSize: Word;
    bInterval: Byte;
end;

Type _USB_CONFIGURATION_DESCRIPTOR = record
    bLength: Byte;
    bDescriptorType: Byte;
    wTotalLength: Word;
    bNumInterfaces: Byte;
    bConfigurationValue: Byte;
    iConfiguration: Byte;
    bmAttributes: Byte;
    MaxPower: Byte;
end;

type _USB_INTERFACE_DESCRIPTOR = record
    bLength: Byte;
    bDescriptorType: Byte;
    bInterfaceNumber: Byte;
    bAlternateSetting: Byte;
    bNumEndpoints: Byte;
    bInterfaceClass: Byte;
    bInterfaceSubClass: Byte;
    bInterfaceProtocol: Byte;
    iInterface: Byte;
end;

type _USB_STRING_DESCRIPTOR = record
    bLength: Byte;
    bDescriptorType: Byte;
    bString: WCHAR; {  bString[1]: WCHAR;}
end;

type _USB_POWER_DESCRIPTOR = record
    bLength: Byte;
    bDescriptorType: Byte;
    bCapabilitiesFlags: Byte;
    EventNotification: Word;
    D1LatencyTime: Word;
    D2LatencyTime: Word;
    D3LatencyTime: Word;
    PowerUnit: Byte;
    D0PowerConsumption: Word;
    D1PowerConsumption: Word;
    D2PowerConsumption: Word;
end;

type _USB_COMMON_DESCRIPTOR = record
    bLength: Byte;
    bDescriptorType: Byte;
end;

type _VENDOR_REQUEST_IN = record
    bRequest : Byte;
    wValue : Word;
    wIndex : Word;
    wLength: Word;
    direction : Byte;
    bData : Byte;
end;

type DEVICE_TYPE = ULONG;

const Ezusb_IOCTL_INDEX  = $0800;

const
 METHOD_BUFFERED    =             0;
 METHOD_IN_DIRECT   =             1;
 METHOD_OUT_DIRECT  =             2;
 METHOD_NEITHER     =             3;

const
 FILE_ANY_ACCESS     =           0;
 FILE_READ_ACCESS    =       $0001;
 FILE_WRITE_ACCESS   =       $0002;

 FILE_DEVICE_UNKNOWN: DEVICE_TYPE =             $0C0000022;

var  IOCTL_Ezusb_GET_PIPE_INFO: Dword;
     IOCTL_Ezusb_GET_DEVICE_DESCRIPTOR: Dword;
     IOCTL_Ezusb_GET_CONFIGURATION_DESCRIPTOR: Dword;
     IOCTL_Ezusb_BULK_OR_INTERRUPT_WRITE: Dword;
     IOCTL_Ezusb_BULK_OR_INTERRUPT_READ: Dword;
     IOCTL_Ezusb_VENDOR_REQUEST: Dword;
     IOCTL_Ezusb_GET_CURRENT_CONFIG: Dword;
     IOCTL_Ezusb_ANCHOR_DOWNLOAD: Dword;
     IOCTL_Ezusb_ISO: Dword;
     IOCTL_Ezusb_RESET: Dword;
     IOCTL_Ezusb_RESETPIPE: Dword;
     IOCTL_Ezusb_ABORTPIPE: Dword;
     IOCTL_Ezusb_SETINTERFACE: Dword;
     IOCTL_Ezusb_GET_STRING_DESCRIPTOR: Dword;
     IOCTL_EZUSB_BULK_READ: Dword;
     IOCTL_EZUSB_BULK_WRITE: Dword;
     IOCTL_EZUSB_GET_CURRENT_FRAME_NUMBER: Dword;
     IOCTL_EZUSB_VENDOR_OR_CLASS_REQUEST: Dword;
     IOCTL_EZUSB_GET_LAST_ERROR: Dword;
     IOCTL_EZUSB_ISO_LOOPBACK: Dword;
     IOCTL_EZUSB_ISO_READ: Dword;
     IOCTL_EZUSB_ISO_WRITE: Dword;
     IOCTL_EZUSB_ANCHOR_DOWNLOAD_2: Dword;
     IOCTL_EZUSB_BULK_LATENCY_TEST: Dword;
     IOCTL_EZUSB_GET_DRIVER_VERSION: Dword;

var  InBuffer: Array [0..8191]of Byte;
     OutBuffer: Array [0..8191]of Byte;

procedure WrRAM(Adr: Word; Dat: Byte);
function RdRAM(Adr: Word): Byte;
procedure WriteRAMbytes(StartAdr, NumberOfBytes: Word);
procedure ReadRAMbytes (StartAdr, NumberOfBytes: Word);
procedure ProgStart();
procedure ProgReset();
procedure DownloadBin(FileName:  String);
procedure DownloadIntelHex (FileName: String);
procedure ReadBulkData (PipeNumber, NumberOfBytes: Word);
procedure WriteBulkData (PipeNumber, NumberOfBytes: Word);
procedure SetInterface (InterfaceNumber, Setting: Byte);


implementation

function CTL_Code (Devicetype: Dword; Func, Method,Access: Word) : Dword;
begin
  CTL_Code := (Devicetype shl 16) Or (Access shl 14) or (Func shl 2) or Method;
end;

procedure WrRAM(Adr: Word; Dat: Byte);       //single Byte
var TemplateHandle: THandle;
    DeviceHandle: THandle;
    bresult: Boolean;
    nBytes: DWord;
    MyRequest: _VENDOR_REQUEST_IN;
begin
   myRequest.bRequest := $A0;
   myRequest.wValue := Adr;
   myRequest.wIndex := 0;
   myRequest.wLength := 1;
   myRequest.direction := 0;
   myRequest.bData := Dat;
   DeviceHandle := CreateFile ('\\.\ezusb-0',Generic_write,File_Share_write,nil,open_existing,0,TemplateHandle);
   bResult := DeviceIoControl(DeviceHandle,IOCTL_Ezusb_VENDOR_REQUEST,@myRequest,10,nil,0,nBytes,nil);
   CloseHandle (DeviceHandle);
end;

function RdRAM(Adr: Word): Byte;
var TemplateHandle: THandle;
    DeviceHandle: THandle;    bresult: Boolean;
    nBytes: DWord;
    MyRequest: _VENDOR_REQUEST_IN;
    n: Integer;
    Buffer: Array [1..2] of Byte;
begin
   myRequest.bRequest := $A0;
   myRequest.wValue := Adr;
   myRequest.wIndex := 0;
   myRequest.wLength := 1;
   myRequest.direction := 1;     // Read
   myRequest.bData := $00;

  DeviceHandle := CreateFile ('\\.\ezusb-0', Generic_write,File_Share_write,nil,open_existing,0,TemplateHandle);
  bResult := DeviceIoControl(DeviceHandle,IOCTL_Ezusb_VENDOR_REQUEST,@myRequest,10,@Buffer,SizeOf(Buffer),nBytes,nil);
  CloseHandle (DeviceHandle);
  RdRAM := Buffer[1];
end;

procedure WriteRAMbytes(StartAdr, NumberOfBytes: Word);
var TemplateHandle: THandle;
    DeviceHandle: THandle;
    bresult: Boolean;
    nBytes: DWord;
    downloadControl: _ANCHOR_DOWNLOAD_CONTROL;
begin
  downloadControl.offset:=StartAdr;
  DeviceHandle := CreateFile ('\\.\ezusb-0',Generic_write,File_Share_write,nil,open_existing,0,TemplateHandle);
  bResult := DeviceIoControl(DeviceHandle,IOCTL_EZUSB_ANCHOR_DOWNLOAD_2,@downloadControl,sizeof(downloadControl),@OutBuffer,NumberOfBytes,nBytes,nil);
  CloseHandle (DeviceHandle);
end;

procedure ReadRAMbytes (StartAdr, NumberOfBytes: Word);
var TemplateHandle: THandle;
    DeviceHandle: THandle;    bresult: Boolean;
    nBytes: DWord;
    MyRequest: _VENDOR_REQUEST_IN;
    n: Integer;
begin
   myRequest.bRequest := $A0;
   myRequest.wValue := StartAdr;
   myRequest.wIndex := 0;
   myRequest.wLength := NumberOfBytes; //max. 1024
   myRequest.direction := 1;     // Read
   myRequest.bData := $00;
  DeviceHandle := CreateFile ('\\.\ezusb-0',
  Generic_write,File_Share_write,nil,open_existing,0,TemplateHandle);
  bResult := DeviceIoControl(DeviceHandle,IOCTL_Ezusb_VENDOR_REQUEST,@myRequest,10,@InBuffer,SizeOf(InBuffer),nBytes,nil);
  CloseHandle (DeviceHandle);
end;

procedure ReadBulkData (PipeNumber, NumberOfBytes: Word);
var TemplateHandle: THandle;
    DeviceHandle: THandle;    bresult: Boolean;
    nBytes: DWord;
    BulkControl: _BULK_TRANSFER_CONTROL;
    n: Integer;
begin
  BulkControl.PipeNum := PipeNumber;
  DeviceHandle := CreateFile ('\\.\ezusb-0',
  Generic_write,File_Share_write,nil,open_existing,0,TemplateHandle);
  bResult := DeviceIoControl(DeviceHandle,IOCTL_EZUSB_BULK_READ,@BulkControl,SizeOf(BulkControl),@InBuffer,NumberOfBytes,nBytes,nil);
  CloseHandle (DeviceHandle);
end;

procedure WriteBulkData (PipeNumber, NumberOfBytes: Word);
var TemplateHandle: THandle;
    DeviceHandle: THandle;    bresult: Boolean;
    nBytes: DWord;
    BulkControl: _BULK_TRANSFER_CONTROL;
    n: Integer;
begin
  BulkControl.PipeNum := PipeNumber;
  DeviceHandle := CreateFile ('\\.\ezusb-0',
  Generic_write,File_Share_write,nil,open_existing,0,TemplateHandle);
  bResult := DeviceIoControl(DeviceHandle,IOCTL_EZUSB_BULK_WRITE,@BulkControl,SizeOf(BulkControl),@OutBuffer,NumberOfBytes,nBytes,nil);
  CloseHandle (DeviceHandle);
end;

procedure SetInterface (InterfaceNumber, Setting: Byte);
var TemplateHandle: THandle;
    DeviceHandle: THandle;    bresult: Boolean;
    nBytes: DWord;
    SetInterface : _Set_INTERFACE_IN;
    n: Integer;
begin
  SetInterface.interfaceNum := InterfaceNumber;
  SetInterface.alternateSetting := Setting;
  DeviceHandle := CreateFile ('\\.\ezusb-0',
  Generic_write,File_Share_write,nil,open_existing,0,TemplateHandle);
  bResult := DeviceIoControl(DeviceHandle,IOCTL_Ezusb_SETINTERFACE,@SetInterface,SizeOf(SetInterface),nil,0,nBytes,nil);
  CloseHandle (DeviceHandle);
end;

procedure ProgStart();
begin
   WrRAM($7F92,0);
end;

procedure ProgReset();
begin
   WrRAM($7F92,1);
end;

function HexToInt (Hex: String): Byte;
var h, l: Byte;
begin
  h:= ord (Hex[1])-48;
  if h>9 then h:=h-7;
  l:= ord (Hex[2])-48;
  if l>9 then l:=l-7;
  HexToInt := 16*h+l;
end;

procedure DownloadBin(FileName:  String);
vAR  f :File of Byte;
     r :Byte;
     Adresse, MaxAdresse, n, i : Word;
     Code: Byte;
begin
  AssignFile(f,FileName);
  {$I-} Reset(f); {$I+}
  r:=IOResult;
  if r = 0 then begin
    Adresse := 0;
    for n:=1 to FileSize(f)do begin;
      Read(f,Code);
      OutBuffer [Adresse] := Code;
      Adresse := Adresse + 1;
    end;
    CloseFile(f);
    WriteRAMbytes (0,Adresse)
  end;
end;

procedure DownloadIntelHex (FileName: String);
 VAR  f :TextFile;
      Line: String;
      r,wert :Byte;
      Num: Byte;
      Adr, MaxAdr, n, i, Code : Word;
begin
    AssignFile(f,FileName);
    {$I-} Reset(f); {$I+}
    r:=IOResult;
    IF r = 0 then begin
      MaxAdr :=0;
      repeat
        Readln(f,line);
       Num := HexToInt (copy(line,2,2));
        Adr := 256*HexToInt(copy(line,4,2))+HexToInt(copy(line,6,2));
        if Num>0 then begin
          for i := 1 to Num do begin
             code := HexToInt(copy(line,8+2*i,2));
             OutBuffer[Adr] := code;
             if Adr > MaxAdr then MaxAdr := Adr;
             Adr := Adr + 1;
          end;
        end;
      until Num = 0;
       CloseFile(f);
     end;
     WriteRAMbytes (0,MaxAdr+1);
  end;

begin
 IOCTL_Ezusb_GET_PIPE_INFO :=    CTL_CODE(FILE_DEVICE_UNKNOWN,
                                                   Ezusb_IOCTL_INDEX+0,
                                                   METHOD_BUFFERED,
                                                   FILE_ANY_ACCESS);;

 IOCTL_Ezusb_GET_DEVICE_DESCRIPTOR := CTL_CODE(FILE_DEVICE_UNKNOWN,
                                                   Ezusb_IOCTL_INDEX+1, 
                                                   METHOD_BUFFERED,
                                                   FILE_ANY_ACCESS);

 IOCTL_Ezusb_GET_CONFIGURATION_DESCRIPTOR := CTL_CODE(FILE_DEVICE_UNKNOWN,
                                                   Ezusb_IOCTL_INDEX+2,
                                                   METHOD_BUFFERED,
                                                   FILE_ANY_ACCESS);

 IOCTL_Ezusb_BULK_OR_INTERRUPT_WRITE :=    CTL_CODE(FILE_DEVICE_UNKNOWN,
                                                   Ezusb_IOCTL_INDEX+3,
                                                   METHOD_BUFFERED,
                                                   FILE_ANY_ACCESS);

 IOCTL_Ezusb_BULK_OR_INTERRUPT_READ :=     CTL_CODE(FILE_DEVICE_UNKNOWN,
                                                   Ezusb_IOCTL_INDEX+4,
                                                   METHOD_BUFFERED,
                                                   FILE_ANY_ACCESS);

 IOCTL_Ezusb_VENDOR_REQUEST :=             CTL_CODE(FILE_DEVICE_UNKNOWN,
                                                   Ezusb_IOCTL_INDEX+5,
                                                   METHOD_BUFFERED,
                                                   FILE_ANY_ACCESS);

 IOCTL_Ezusb_GET_CURRENT_CONFIG :=         CTL_CODE(FILE_DEVICE_UNKNOWN,
                                                   Ezusb_IOCTL_INDEX+6,
                                                   METHOD_BUFFERED,
                                                   FILE_ANY_ACCESS);

 IOCTL_Ezusb_ANCHOR_DOWNLOAD :=            CTL_CODE(FILE_DEVICE_UNKNOWN,
                                                   Ezusb_IOCTL_INDEX+7,
                                                   METHOD_BUFFERED,
                                                   FILE_ANY_ACCESS);

 IOCTL_Ezusb_ISO :=            CTL_CODE(FILE_DEVICE_UNKNOWN,
                                                   Ezusb_IOCTL_INDEX+10,
                                                   METHOD_BUFFERED,
                                                   FILE_ANY_ACCESS);

 IOCTL_Ezusb_RESET := CTL_CODE(FILE_DEVICE_UNKNOWN,
                                                   Ezusb_IOCTL_INDEX+12,
                                                   METHOD_IN_DIRECT,
                                                   FILE_ANY_ACCESS);

 IOCTL_Ezusb_RESETPIPE := CTL_CODE(FILE_DEVICE_UNKNOWN,
                                                   Ezusb_IOCTL_INDEX+13,
                                                   METHOD_IN_DIRECT,
                                                   FILE_ANY_ACCESS);

 IOCTL_Ezusb_ABORTPIPE := CTL_CODE(FILE_DEVICE_UNKNOWN,
                                                   Ezusb_IOCTL_INDEX+15,
                                                   METHOD_IN_DIRECT,
                                                   FILE_ANY_ACCESS);

 IOCTL_Ezusb_SETINTERFACE := CTL_CODE(FILE_DEVICE_UNKNOWN,
                                                   Ezusb_IOCTL_INDEX+16,
                                                   METHOD_BUFFERED,
                                                   FILE_ANY_ACCESS);

 IOCTL_Ezusb_GET_STRING_DESCRIPTOR := CTL_CODE(FILE_DEVICE_UNKNOWN,
                                                   Ezusb_IOCTL_INDEX+17,
                                                   METHOD_BUFFERED,
                                                   FILE_ANY_ACCESS);

 IOCTL_EZUSB_BULK_READ :=            CTL_CODE(FILE_DEVICE_UNKNOWN,
                                                   Ezusb_IOCTL_INDEX+19,
                                                   METHOD_OUT_DIRECT,
                                                   FILE_ANY_ACCESS);

 IOCTL_EZUSB_BULK_WRITE :=           CTL_CODE(FILE_DEVICE_UNKNOWN,
                                                   Ezusb_IOCTL_INDEX+20,
                                                   METHOD_IN_DIRECT,
                                                   FILE_ANY_ACCESS);

 IOCTL_EZUSB_GET_CURRENT_FRAME_NUMBER := CTL_CODE(FILE_DEVICE_UNKNOWN,
                                                   Ezusb_IOCTL_INDEX+21,
                                                   METHOD_BUFFERED,
                                                   FILE_ANY_ACCESS);


 IOCTL_EZUSB_VENDOR_OR_CLASS_REQUEST :=  CTL_CODE(FILE_DEVICE_UNKNOWN,
                                                   Ezusb_IOCTL_INDEX+22,
                                                   METHOD_IN_DIRECT,
                                                   FILE_ANY_ACCESS);


 IOCTL_EZUSB_GET_LAST_ERROR :=  CTL_CODE(FILE_DEVICE_UNKNOWN,
                                                   Ezusb_IOCTL_INDEX+23,
                                                   METHOD_BUFFERED,
                                                   FILE_ANY_ACCESS);

 IOCTL_EZUSB_ISO_LOOPBACK :=         CTL_CODE(FILE_DEVICE_UNKNOWN,
                                                   Ezusb_IOCTL_INDEX+24,
                                                   METHOD_IN_DIRECT,
                                                   FILE_ANY_ACCESS);

 IOCTL_EZUSB_ISO_READ :=         CTL_CODE(FILE_DEVICE_UNKNOWN,
                                                   Ezusb_IOCTL_INDEX+25,
                                                   METHOD_OUT_DIRECT,
                                                   FILE_ANY_ACCESS);

 IOCTL_EZUSB_ISO_WRITE :=         CTL_CODE(FILE_DEVICE_UNKNOWN,
                                                   Ezusb_IOCTL_INDEX+26,
                                                   METHOD_IN_DIRECT,
                                                   FILE_ANY_ACCESS);

 IOCTL_EZUSB_ANCHOR_DOWNLOAD_2 :=  CTL_CODE(FILE_DEVICE_UNKNOWN,
                                                   Ezusb_IOCTL_INDEX+27,
                                                   METHOD_IN_DIRECT,
                                                   FILE_ANY_ACCESS);

 IOCTL_EZUSB_BULK_LATENCY_TEST := CTL_CODE(FILE_DEVICE_UNKNOWN,
                                                   Ezusb_IOCTL_INDEX+28,
                                                   METHOD_BUFFERED,
                                                   FILE_ANY_ACCESS);

 IOCTL_EZUSB_GET_DRIVER_VERSION :=  CTL_CODE(FILE_DEVICE_UNKNOWN,
                                                   Ezusb_IOCTL_INDEX+29,
                                                   METHOD_BUFFERED,
                                                   FILE_ANY_ACCESS);
end.
